
{  }
{                                                                        }
{      Virtual Pascal Runtime Library.  Version 2.1                      }
{      Windows shell routines                                            }
{      (C) 2001, Tramontana Co. Released into the public domain          }
{      2003.01.28: IShellLinkDataList support added by Pierre Muller     }
{      2005.10.25: Adapted to VP by LPJJl Software - lpjjl@skynet.be     }
{      -------------------------------------------------------------     }
{      Copyright (C) 1999-2005 vpascal.com                               }
{                                                                        }
{  }

Unit WinShell;

INTERFACE

Uses
  Windows;

Type
  PGUID = ^TGUID;
  TGUID = Packed Record
            Data1 : DWord;
            Data2 : SmallWord;
            Data3 : SmallWord;
            Data4 : Array [0..7] of Byte;
          end;
  PIID = PGUID;
  TIID = TGUID;
  CLSID = TGUID;
  LPCLSID = ^CLSID;
  TCLSID = CLSID;
  PCLSID = ^CLSID;

  SHITEMID = Record
               cb : USHORT;
               abID : Array [0..0] of Byte;
             end;
  LPSHITEMID = ^SHITEMID;
  LPCSHITEMID = ^SHITEMID;
  _SHITEMID = SHITEMID;
  TSHITEMID = SHITEMID;
  PSHITEMID = ^SHITEMID;

  ITEMIDLIST = Record
                 mkid : SHITEMID;
               end;
  LPITEMIDLIST = ^ITEMIDLIST;
  LPCITEMIDLIST = ^ITEMIDLIST;
  _ITEMIDLIST = ITEMIDLIST;
  TITEMIDLIST = ITEMIDLIST;
  PITEMIDLIST = ^ITEMIDLIST;

  TCOORD = Record
             X : SHORT;
             Y : SHORT;
           end;

Const
  { Execution context constants }
  CLSCTX_INPROC_SERVER   = 1;
  CLSCTX_INPROC_HANDLER  = 2;
  CLSCTX_LOCAL_SERVER    = 4;
  CLSCTX_INPROC_SERVER16 = 8;
  CLSCTX_REMOTE_SERVER   = 16;
  CLSCTX_SERVER          = CLSCTX_INPROC_SERVER or CLSCTX_LOCAL_SERVER or CLSCTX_REMOTE_SERVER;
  CLSCTX_ALL             = CLSCTX_INPROC_HANDLER or CLSCTX_SERVER;
  { SHGetSpecialFolder... constants }
  CSIDL_DESKTOP                 = $0000;
  CSIDL_INTERNET                = $0001;
  CSIDL_PROGRAMS                = $0002;
  CSIDL_CONTROLS                = $0003;
  CSIDL_PRINTERS                = $0004;
  CSIDL_PERSONAL                = $0005;
  CSIDL_FAVORITES               = $0006;
  CSIDL_STARTUP                 = $0007;
  CSIDL_RECENT                  = $0008;
  CSIDL_SENDTO                  = $0009;
  CSIDL_BITBUCKET               = $000A;
  CSIDL_STARTMENU               = $000B;
  CSIDL_DESKTOPDIRECTORY        = $0010;
  CSIDL_DRIVES                  = $0011;
  CSIDL_NETWORK                 = $0012;
  CSIDL_NETHOOD                 = $0013;
  CSIDL_FONTS                   = $0014;
  CSIDL_TEMPLATES               = $0015;
  CSIDL_COMMON_STARTMENU        = $0016;
  CSIDL_COMMON_PROGRAMS         = $0017;
  CSIDL_COMMON_STARTUP          = $0018;
  CSIDL_COMMON_DESKTOPDIRECTORY = $0019;
  CSIDL_APPDATA                 = $001A;
  CSIDL_PRINTHOOD               = $001B;
  CSIDL_ALTSTARTUP              = $001D;
  CSIDL_COMMON_ALTSTARTUP       = $001E;
  CSIDL_COMMON_FAVORITES        = $001F;
  CSIDL_INTERNET_CACHE          = $0020;
  CSIDL_COOKIES                 = $0021;
  CSIDL_HISTORY                 = $0022;
  { Various GUIDs/CLSIDs }
  CLSID_ShellDesktop : TGUID     = (Data1 : $00021400; Data2 : 0; Data3 : 0; Data4 : ($C0, 0, 0, 0, 0, 0, 0, $46));
  CLSID_ShellLink : TGUID        = (Data1 : $00021401; Data2 : 0; Data3 : 0; Data4 : ($C0, 0, 0, 0, 0, 0, 0, $46));
  IID_IShellLink : TGUID         = (Data1 : $000214EE; Data2 : 0; Data3 : 0; Data4 : ($C0, 0, 0, 0, 0, 0, 0, $46));
  IID_IShellLinkDataList : TGUID = (Data1 : $45E2B4AE; Data2 : $B1C3; Data3 : $11D0; Data4 : ($B9, $2f, 0, $A0, $C9, $03, $12, $E1));
  IID_IPersistFile : TGUID       = (Data1 : $0000010B; Data2 : 0; Data3 : 0; Data4 : ($C0, 0, 0, 0, 0, 0, 0, $46));
  { IShellLink.Resolve flags }
  SLR_NO_UI     = $00020001;
  SLR_ANY_MATCH = $0002;
  SLR_UPDATE    = $0004;
  SLR_NOUPDATE  = $0008;

  STGM_READ      = $00000000;
  STGM_WRITE     = $00000001;
  STGM_READWRITE = $00000002;
  { IShellLink.GetPath flags }
  SLGP_SHORTPATH   = $0001;
  SLGP_UNCPRIORITY = $0002;
  SLGP_RAWPATH     = $0004;

Type
  { COM interfaces }
  { IUnknown }
  PIUnknown = ^IUnknown;
  PIUnknownVtbl = ^IUnknownVtbl;
  IUnknown = Packed Record
               Vtbl : PIUnknownVtbl;
             end;
  IUnknownVtbl = Packed Record
                   {&StdCall+}
                   QueryInterface : Function (const this : PIUnknown; const iid: TIID; var obj): HResult;
                   AddRef : Function (const this : PIUnknown) : LongInt;
                   Release : Function (const this : PIUnknown) : LongInt;
                   {&StdCall-}
                 end;
  { IMalloc }
  PPIMalloc = ^PIMalloc;
  PIMalloc = ^IMalloc;
  PIMallocVtbl = ^IMallocVtbl;
  IMalloc = Packed Record
              Vtbl : PIMallocVtbl;
            end;
  IMallocVtbl = Packed Record
                  {&StdCall+}
                  QueryInterface : Function (const this : PIMalloc; const iid: TIID; var obj): HResult;
                  AddRef : Function (const this : PIMalloc) : ULONG;
                  Release : Function (const this : PIMalloc) : ULONG;
                  Alloc : Function (const this : PIMalloc; cb : ULONG) : Pointer;
                  Realloc : Function (const this : PIMalloc; var pv; cb : ULONG) : Pointer;
                  Free : Procedure (const this : PIMalloc; pv : Pointer);
                  GetSize : Function (const this : PIMalloc; pv : Pointer) : ULONG;
                  DidAlloc : Function (const this : PIMalloc; pv : Pointer) : INT;
                  HeapMinimize : Procedure (const this : PIMalloc);
                  {&StdCall-}
                end;
  { IShellLink }
  PIShellLink = ^IShellLink;
  PIShellLinkVtbl = ^IShellLinkVtbl;
  IShellLink = Packed Record
                 Vtbl : PIShellLinkVtbl;
               end;
  IShellLinkVtbl = Packed Record
                     {&StdCall+}
                     QueryInterface : Function (const this : PIShellLink; const iid: TIID; var obj): HResult;
                     AddRef : Function (const this : PIShellLink) : ULONG;
                     Release : Function (const this : PIShellLink) : ULONG;
                     GetPath : Function (const this : PIShellLink; pszFile : LPSTR; cchMaxPAth : INT; var fd : TWin32FindData; Flags : DWORD) : hResult;
                     GetIDList : Function (const this : PIShellLink; var pidl : LPITEMIDLIST) : hResult;
                     SetIDList : Function (const this : PIShellLink; pidl : LPITEMIDLIST) : hResult;
                     GetDescription : Function (const this : PIShellLink; pszName : LPSTR; cchMaxName : INT) : hResult;
                     SetDescription : Function (const this : PIShellLink; pszName : LPSTR) : hResult;
                     GetWorkingDirectory : Function (const this : PIShellLink; pszDir : LPSTR; cchMaxName : INT) : hResult;
                     SetWorkingDirectory : Function (const this : PIShellLink; pszDir : LPSTR) : hResult;
                     GetArguments : Function (const this : PIShellLink; pszArgs : LPSTR; cchMaxName : INT) : hResult;
                     SetArguments : Function (const this : PIShellLink; pszArgs : LPSTR) : hResult;
                     GetHotkey : Function (const this : PIShellLink; var wHotKey : Word) : hResult;
                     SetHotkey : Function (const this : PIShellLink; wHotKey : Word) : hResult;
                     GetShowCmd : Function (const this : PIShellLink; var iShowCmd : INT) : hResult;
                     SetShowCmd : Function (const this : PIShellLink; iShowCmd : INT) : hResult;
                     GetIconLocation : Function (const this : PIShellLink; pszIconPath : LPSTR; cchIconPath : INT; var iIcon : INT) : hResult;
                     SetIconLocation : Function (const this : PIShellLink; pszIconPath : LPSTR; iIcon : INT) : hResult;
                     SetRelativePath : Function (const this : PIShellLink; pszPathRel : LPSTR; wReserved : DWORD) : hResult;
                     Resolve : Function (const this : PIShellLink; hwnd : HWND; fFlags : DWORD) : hResult;
                     SetPath : Function (const this : PIShellLink; pszFile : LPSTR) : hResult;
                     {&StdCall-}
                   end;
  { IPersistFile }
  PIPersistFile = ^IPersistFile;
  PIPersistFileVtbl = ^IPersistFileVtbl;
  IPersistFile = Packed Record
                   Vtbl : PIPersistFileVtbl;
                 end;
  IPersistFileVtbl = Packed Record
                       {&StdCall+}
                       QueryInterface : Function (const this : PIPersistFile; const iid: TIID; var obj): HResult;
                       AddRef : Function (const this : PIPersistFile) : ULONG;
                       Release : Function (const this : PIPersistFile) : ULONG;
                       GetClassID : Function (const this : PIPersistFile; ClassID : TCLSID) : hResult;
                       IsDirty : Function (const this : PIPersistFile) : hResult;
                       Load : Function (const this : PIPersistFile; plszFilename : LPWSTR; dwMode : DWORD) : hResult;
                       Save : Function (const this : PIPersistFile; plszFilename : LPWSTR; fRemember : BOOL) : hResult;
                       SaveCompleted : Function (const this : PIPersistFile; plszFilename : LPWSTR) : hResult;
                       GetCurFile : Function (const this : PIPersistFile; var plszFilename : LPWSTR) : hResult;
                       {&StdCall-}
                     end;

{ IShellLinkDataList }
{ inplemented in shell32.dll version 4.71 or later }

{Data block structureDescription}
Const
  EXP_DARWIN_ID_SIG        = $A0000006;   { The link's Microsoft Windows Installer identifier (ID). }
  EXP_LOGO3_ID_SIG         = $A0000007;
  EXP_SPECIAL_FOLDER_SIG   = $A0000005;   { Special folder information. }
  EXP_SZ_LINK_SIG          = $A0000001;   { The target name. }
  EXP_SZ_ICON_SIG          = $A0000007;   { The icon name. }
  NT_CONSOLE_PROPS_SIG     = $A0000002;   { Console properties. }
  NT_FE_CONSOLE_PROPS_SIG  = $A0000004;   { The console's code page. }

  SLDF_HAS_ID_LIST         = $00000001;   { The link has ID list. }
  SLDF_HAS_LINK_INFO       = $00000002;   { The link has LinkInfo. }
  SLDF_HAS_NAME            = $00000004;   { The link has a name. }
  SLDF_HAS_RELPATH         = $00000008;   { The link has a relative path. }
  SLDF_HAS_WORKINGDIR      = $00000010;   { The link has a working directory. }
  SLDF_HAS_ARGS            = $00000020;   { The link has arguments. }
  SLDF_HAS_ICONLOCATION    = $00000040;   { The link has an icon location. }
  SLDF_UNICODE             = $00000080;   { The strings are unicode. }
  SLDF_FORCE_NO_LINKINFO   = $00000100;   { Do not create link information. Distributed tracking will be disabled. }
  SLDF_HAS_EXP_SZ          = $00000200;   { The link contains expandable environment strings. }
  SLDF_RUN_IN_SEPARATE     = $00000400;   { Run the 16-bit target exe in a separate VDM/WOW. }
  SLDF_HAS_LOGO3ID         = $00000800;   { The link is a special Logo3/MSICD link. }
  SLDF_HAS_DARWINID        = $00001000;   { The link is a special Darwin link. }
  SLDF_RUNAS_USER          = $00002000;   { Run the link as a different user. }
  SLDF_HAS_EXP_ICON_SZ     = $00004000;   { The link contains expandable env string for icon path. }
  SLDF_NO_PIDL_ALIAS       = $00008000;   { Don't ever resolve to a logical location. }
  SLDF_FORCE_UNCNAME       = $00010000;   { Make GetPath() prefer the UNC name to the local name. }
  SLDF_RUN_WITH_SHIMLAYER  = $00020000;   { Launch the target of this link w/ shim layer active. }
  SLDF_RESERVED            = $80000000;   { Reserved-- so we can use the low Word as an index value in the future }

Type

  DATABLOCK_HEADER = Packed Record
                       cbSize,
                       dwSignature : DWORD;
                     end;

  EXP_DARWIN_LINK = Packed Record
                      dbh : DATABLOCK_HEADER;
                      szDarwinID : Array [0..MAX_PATH-1] of Char;
                      szwDarwinID : Array [0..MAX_PATH-1] of Word;
                    end;

  EXP_SPECIAL_FOLDER = Packed Record
                         dbh : DATABLOCK_HEADER;
                         idSpecialFolder,
                         cbOffset : DWORD;
                       end;

  EXP_SZ_LINK = Packed Record
                  dbh : DATABLOCK_HEADER;
                  szTarget : Array [0..MAX_PATH-1] of Char;
                  szwTarget : Array [0..MAX_PATH-1] of Word;
                end;

  NT_CONSOLE_PROPS = Packed Record
                       dbh : DATABLOCK_HEADER;
                       wFillAttribute : Word;
                       wPopupFillAttribute : Word;
                       dwScreenBufferSize : TCOORD;
                       dwWindowSize : TCOORD;
                       dwWindowOrigin : TCOORD;
                       nFont : DWORD;
                       nInputBufferSize : DWORD;
                       dwFontSize : TCOORD;
                       uFontFamily : UINT;
                       uFontWeight : UINT;
                       FaceName : Array [0..LF_FACESIZE - 1] of Word;
                       uCursorSize : UINT;
                       bFullScreen : BOOL;
                       bQuickEdit : BOOL;
                       bInsertMode : BOOL;
                       bAutoPosition : BOOL;
                       uHistoryBufferSize : UINT;
                       uNumberOfHistoryBuffers : UINT;
                       bHistoryNoDup : BOOL;
                       ColorTable : Array [0..16 - 1] of COLORREF;
                     end;

  NT_FE_CONSOLE_PROPS = Packed Record
                          dbh : DATABLOCK_HEADER;
                          uCodePage : UINT;
                        end;

{ IShellLinkDataList }
  PIShellLinkDataList = ^IShellLinkDataList;
  PIShellLinkDataListVtbl = ^IShellLinkDataListVtbl;
  IShellLinkDataList = Packed Record
                         Vtbl : PIShellLinkDataListVtbl;
                       end;
  IShellLinkDataListVtbl = Packed Record
                             {&StdCall+}
                             QueryInterface : Function (const this : PIShellLinkDataList; const iid: TIID; var obj): HResult;
                             AddRef : Function (const this : PIShellLinkDataList) : ULONG;
                             Release : Function (const this : PIShellLinkDataList) : ULONG;
                             AddDataBlock : Function (const this : PIShellLinkDataList;PDataBlock : Pointer) : ULONG;
                                (* Adds a data block to a link *)
                             CopyDataBlock : Function (const this : PIShellLinkDataList;dwSig : ULONG;var pDataBlock : Pointer) : ULONG;
                                (* Retrieves a copy of a link's data block - pDataBlock must be freed with LocalFree *)
                             RemoveDataBlock : Function (const this : PIShellLinkDataList;dwSig : DWORD) : HResult;
                                (* Removes a data block from a link *)
                             GetFlags : Function (const this : PIShellLinkDataList;var dwFlags : DWORD) : HResult;
                                (* Retrieves the current option settings *)
                             SetFlags : Function (const this : PIShellLinkDataList;dwFlags : DWORD) : HResult;
                                (* Specifies the current option settings *)
                             {&StdCall-}
                           end;

{&StdCall+}
Function CoInitialize (pvReserved : Pointer) : hResult;

Procedure CoUninitialize;

Function SHGetMalloc (ppMalloc : PPIMalloc) : hResult;

Function CoCreateInstance (rclsid : TCLSID; pUnkOuter : PIUnknown; dwClsContext : LongInt; riid : TIID; var ppv) : hResult;

Function SHGetSpecialFolderLocation (_p1 : HWND; _p2 : LongInt; var _p3 : LPITEMIDLIST) : hResult;

Function SHGetPathFromIDList (_p1 : LPITEMIDLIST; _p2 : LPTSTR) : LONGBOOL;
{&StdCall-}


IMPLEMENTATION

Function CoInitialize (pvReserved : Pointer) : hResult; external 'ole32' name 'CoInitialize';

Procedure CoUninitialize; external 'ole32' name 'CoUninitialize';

Function SHGetMalloc (ppMalloc : PPIMalloc) : hResult; external 'shell32' name 'SHGetMalloc';

Function CoCreateInstance (rclsid : TCLSID; pUnkOuter : PIUnknown; dwClsContext : LongInt; riid : TIID; var ppv) : hResult; external 'ole32' name 'CoCreateInstance';

Function SHGetSpecialFolderLocation (_p1 : HWND; _p2 : LongInt; var _p3 : LPITEMIDLIST) : hResult; external 'shell32' name 'SHGetSpecialFolderLocation';

Function SHGetPathFromIDList (_p1 : LPITEMIDLIST; _p2 : LPTSTR) : LONGBOOL; external 'shell32' name 'SHGetPathFromIDList';


END.

